#include <stdio.h>
#include <stdlib.h>
#include "tx_api.h"
#include "naflash.h"
#include <Flash.h>
#include "sockapi.h"
#include "fservapi.h"
#include "sysAccess.h"
#include "bsp_api.h"
#include "reg_def.h"
#ifdef BSP_ARM9
//#include "cs.h"
#include "gpiomux_def.h"
#endif
#include "flashftp.h"

// from bsp.h
#define	BSP_NVRAM_NONE						0
#define	BSP_NVRAM_EEPROM					1
#define	BSP_NVRAM_LAST_FLASH_SECTOR			2
#define	BSP_NVRAM_SEEPROM					3
#define	BSP_NVRAM_SEEPROM_WITH_SEMAPHORES	4
#define	BSP_NVRAM_CUSTOM					255

#define BUFSIZE	(64 * 1024)
static char				s_buf[BUFSIZE];
static unsigned int		s_off;
static unsigned int		s_abs_off;
static unsigned int		s_cur_sector;
static char				s_sysname[256];
static unsigned int		s_romsize;
static int				s_sectors;
static unsigned long*	s_sector_sizes;
static unsigned int		s_available_romsize;
static int				s_available_sectors;
static char				s_errorlog[40];

static int ftp_retr(char *argp, unsigned long handle)
{
    char buffer[64];
	memset(buffer, 0, sizeof(buffer));
	s_abs_off = s_cur_sector = s_off = 0;
	if (!strcmp(argp, "id"))	{
		memset(s_errorlog, 0, sizeof(s_errorlog));
		if (s_romsize == 0x100000)	{
			sprintf(buffer, "NETARM-FTPDL-V100");
		}
		else if (s_romsize == 0x200000)	{
			sprintf(buffer, "NETARM-FTPDL-V200");
		}
		else	{
			sprintf(buffer, "NETARM-FTPDL-V400");
		}
	}
	else if (!strcmp(argp, "errmsg"))	{
		sprintf(buffer, s_errorlog);
	}
	else	{
		strncpy(buffer, argp, 63);
	}
	send(FSHandleToSocket(handle), buffer, strlen(buffer), 0);
    return 0;
}

static int init_sectors_info()
{
	int	result;
	//	get Flash sectors info.
	//	s_sectors = 35
	//	s_sector_size[0] = 16K
	//	s_sector_size[1] = 8K
	//	s_sector_size[2] = 8K
	//	s_sector_size[3] = 32K
	//	s_sector_size[4...34] = 64K
	//	amount = 2M
	//	NAAppOffsetInFlash = 64K
    s_sectors = NAFlashSectors();
    s_sector_sizes = (unsigned long *)malloc(sizeof(unsigned long) * s_sectors);
	result = NAFlashSectorSizes(s_sector_sizes);
	if (result != NAFLASH_SUCCESS)	{
		return result;
    }

	//	get flash size (=2M)
	s_romsize = 0;
	int	i;
	for (i = 0; i < s_sectors; i++)	{
		s_romsize += s_sector_sizes[i];
	}

	// for protect nvram
	s_available_romsize = s_romsize;
	s_available_sectors = s_sectors;
	if (NABspNvramDriver == BSP_NVRAM_LAST_FLASH_SECTOR)	{
		s_available_romsize -= s_sector_sizes[--s_available_sectors];
	}

	return NAFLASH_SUCCESS;
}

static int absoffset2sectoroffset(unsigned int abs_offset, unsigned int* sector, unsigned int* offset)
{
	int	i = 0;
	while (1)	{
		if (abs_offset < s_sector_sizes[i])	{
			*sector = i;
			*offset = abs_offset;
			break;
		}
		abs_offset -= s_sector_sizes[i];
		if (++i >= s_available_sectors)	return NAFLASH_SECTORNUMBER_INVALID;
	}
	return NAFLASH_SUCCESS;
}

static int ftp_stor(char* bufferp, int buflen, char* argp, unsigned long handle)
{
	if (!strcmp(argp, "rom.bin") || !strcmp(argp, "spi_rom.bin") || !strcmp(argp, "romzip.bin") || !strcmp(argp, "image.bin"))	{
		if (!strcmp (argp, "image.bin") && s_cur_sector == 0)	{
			//	first data block of image.bin
			unsigned int	sector, offset;
			if (absoffset2sectoroffset(NAAppOffsetInFlash, &sector, &offset) != NAFLASH_SUCCESS)	{
				//	NAAppOffsetInFlash is set incorrectly.
				return (-2);
			}
			if (offset != 0)	{
				//	The application image must start on a sector boundary.
				return (-2);
			}
			s_cur_sector = sector;
			s_abs_off = NAAppOffsetInFlash;
		}

		if (s_abs_off + buflen > s_available_romsize)	{
			//	Image size exceeds flash memory size.
			s_abs_off = s_cur_sector = s_off = 0;
			return (-2);
		}

		while (buflen)	{
			unsigned int	copysize = s_sector_sizes[s_cur_sector] - s_off;
			if (buflen < copysize)	copysize = buflen;
			memcpy(&s_buf[s_off], bufferp, copysize);
			s_off += copysize;
			s_abs_off += copysize;
			buflen -= copysize;
			bufferp += copysize;

			if (s_off == s_sector_sizes[s_cur_sector])	{
				//	write flash
				if (s_cur_sector < s_available_sectors)	{
					int	result = NAFlashWrite(s_cur_sector, 0, s_off, s_buf, ERASE_AS_NEEDED);
					if (result)	{
						s_abs_off = s_cur_sector = s_off = 0;
						sprintf(s_errorlog, "FLASH WRITE ERR:%2d\n", result);
						return result;
					}
					s_cur_sector++;
					s_off = 0;
				}
			}
		}
	}
	return 0;
}

TX_TIMER resetTimer;

static void ftp_restart(ULONG notUsed)
{
    NA_UNUSED_PARAM(notUsed);
    customizeReset();
}

static int ftp_close(char *argp, unsigned long handle)
{
	if (s_off)	{
		//	write flash
		if (s_cur_sector < s_available_sectors)	{
			int	result = NAFlashWrite(s_cur_sector, 0, s_off, s_buf, ERASE_AS_NEEDED);
			if (result)	{
				s_abs_off = s_cur_sector = s_off = 0;
				sprintf(s_errorlog, "FLASH WRITE ERR:%2d\n", result);
				return result;
			}
			s_cur_sector++;
			s_off = 0;
		}
	}

	if (s_cur_sector)	{
		s_abs_off = s_cur_sector = s_off = 0;
		tx_timer_create(&resetTimer, "ResetTimer", ftp_restart, 0, NABspTicksPerSecond*10, 0, TX_AUTO_ACTIVATE);
	}
	return 0;
}

static int ftp_syst(char* fsParameters, char* fsUserData, unsigned long handle)
{
	NA_UNUSED_PARAM(handle);
	NA_UNUSED_PARAM(fsParameters);
	strncpy(fsUserData, s_sysname, 255);
	return 0;
}

int	flashftp_start(char* sysname)
{
	int result;

	//	initialize sectors info and romsize.
	result = init_sectors_info();
	if (result != NAFLASH_SUCCESS)	{
		return result;
	}

	//	2 connections at same time.
	FSInitialize(2); 

	//	set callbacks.
	FSRegisterSTOR(ftp_stor);
	FSRegisterRETR(ftp_retr);
	FSRegisterControlClose(ftp_close);

	//	set system name
	strncpy(s_sysname, sysname, 255);
	result = FSRegisterSYST(ftp_syst);
    if (result != NAFTPS_SUCCESS)	{
		return result;
    }

	//	setup params.
	s_abs_off = s_cur_sector = s_off = 0;
	memset(s_errorlog, 0, sizeof(s_errorlog));

	//	start FTP server.
	return FSStartServer();
}
