#include "net.h"

#include <stdlib.h>
#include <tx_api.h>
#include <boardParams.h>

int	is_network_linking()
{
	return customizeMiiCheckLink();
}

typedef enum
{
	naIpStateNone,
	naIpStateInitialized,
	naIpStateRestarting,
	naIpStateConfigured
}	naIpStateType;

int	wait_until_network_ip_configured(unsigned long timeout)
{
	int	result = -1;
	if (NAWaitForIpConfig == TRUE)	{
		/* wait until stack has IP parameters */
		int	time = 0;
		while (1)	{
			if (NAGetIPConfigureState() == naIpStateConfigured)	{
				result = 0;
				break;
			}
			if (timeout != 0 && time >= timeout)	{
				break;
			}
			tx_thread_sleep(1);    /* let other tasks run */
			time++;
		}
	}
	return result;
}


static configAceStaticInfo	static_config;
static configAceDhcpInfo	dhcp_config;

void	get_static_ip(unsigned long* addr, unsigned long* mask, unsigned long* gateway, unsigned long* dns0, unsigned long* dns1, unsigned long* dns2)
{
	customizeAceGetStaticConfig(BP_ETH_INTERFACE, &static_config);
	if (addr)		*addr		= static_config.ip_address;
	if (mask)		*mask		= static_config.subnet_mask;
	if (gateway)	*gateway	= static_config.gateway;
	if (dns0)		*dns0		= static_config.name_server_address[0];
	if (dns1)		*dns1		= static_config.name_server_address[1];
	if (dns2)		*dns2		= static_config.name_server_address[2];
}

int	set_static_ip(unsigned long addr, unsigned long mask, unsigned long gateway, unsigned long dns0, unsigned long dns1, unsigned long dns2)
{
	customizeStopAce(BP_ETH_INTERFACE);
	customizeAceGetStaticConfig(BP_ETH_INTERFACE, &static_config);
	static_config.isEnabled					= TRUE;
	static_config.ip_address				= addr;
	static_config.subnet_mask				= mask;
	static_config.gateway					= gateway;
	static_config.name_server_address[0]	= dns0;
	static_config.name_server_address[1]	= dns1;
	static_config.name_server_address[2]	= dns2;
	static_config.auto_assign 				= TRUE;
	customizeAceSetStaticConfig(BP_ETH_INTERFACE, &static_config);
	customizeAceGetDhcpConfig(BP_ETH_INTERFACE, &dhcp_config);
	if (dhcp_config.isEnabled)	{
		dhcp_config.isEnabled = FALSE;
		customizeAceSetDhcpConfig(BP_ETH_INTERFACE, &dhcp_config);
	}
	customizeStartAce();
	return 0;
}

int	set_dhcp_ip()
{
	customizeStopAce(BP_ETH_INTERFACE);
	customizeAceGetStaticConfig(BP_ETH_INTERFACE, &static_config);
	static_config.isEnabled = FALSE;
	static_config.auto_assign = FALSE;
	customizeAceSetStaticConfig(BP_ETH_INTERFACE, &static_config);
	customizeAceGetDhcpConfig(BP_ETH_INTERFACE, &dhcp_config);
	if (!dhcp_config.isEnabled)	{
		dhcp_config.isEnabled = TRUE;
		customizeAceSetDhcpConfig(BP_ETH_INTERFACE, &dhcp_config);
	}
	customizeStartAce();
}

#define	MAXSRESULT	10
static int	sindex = 0;
static char	sresult[MAXSRESULT][20];

char*	ip2s(unsigned long ip)
{
	char*	s = sresult[sindex];
	if (++sindex == MAXSRESULT)	sindex = 0;
	NAInet_toa(ip, s);
	return s;
}

char*	mac2s(unsigned char* mac)
{
	char*	s = sresult[sindex];
	if (++sindex == MAXSRESULT)	sindex = 0;
	sprintf(s, "%02X:%02X:%02X:%02X:%02X:%02X", mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
	return s;
}
